import 'package:project_core/core/exception/AppException.dart';
import 'package:project_core/core/model/AppResponse.dart';
import 'package:project_core/core/model/error/ApiError.dart';
import 'package:rxdart/rxdart.dart';

typedef S Action<S>();

abstract class Bloc {
  void initialize() {}

  execute(BehaviorSubject controller, Action action,
      {bool isLoading = true, bool isError = true}) async {
    try {
      if (isLoading) {
        controller.add(AppResponse.loading());
      }
      await action();
    } on AppException catch (e) {
      if (isError) {
        controller.add(AppResponse.error(e.toString()));
      }
    } on BadRequestException catch (e) {
      ApiError error = ApiError.fromJson(e.data);
      if (isError) {
        controller.add(AppResponse.error(error));
      }
    } catch (e) {
      if (isError) {
        controller.add(AppResponse.error(
            "Não conseguimos realizar sua requisição: " + e.toString()));
      }
    }
  }

  void dispose();
}
