import 'dart:io';
import 'dart:async';

import 'package:connectivity/connectivity.dart';
import 'package:project_core/core/exception/AppException.dart';
import 'package:project_core/core/http/AppHttpResponse.dart';
import 'package:project_core/core/model/Entity.dart';
import 'package:project_core/core/provider/Provider.dart';

typedef S ClientRequest<S>();

abstract class RestProvider<T extends Entity> extends Provider<T> {
  String _apiUrl;

  RestProvider(String apiUrl) {
    _apiUrl = apiUrl;
  }

  dynamic getClient();

  AppHttpResponse createHttpResponse(dynamic response);

  Future<dynamic> fetchAll() async {
    return await fetch(() => getClient().get(_apiUrl));
  }

  Future<dynamic> fetchById(dynamic id) async {
    return await fetch(() => getClient().get("$_apiUrl/$id"));
  }

  Future<dynamic> delete(dynamic id) async {
    return await fetch(() => getClient().delete("$_apiUrl/$id"));
  }

  Future<dynamic> update(T instance) async {
    return await fetch(() =>
        getClient()
            .put("$_apiUrl/${instance.getId()}", data: instance.toJson()));
  }

  Future<dynamic> insert(T instance) async {
    return await fetch(
            () => getClient().post("$_apiUrl", data: instance.toJson()));
  }

  Future<dynamic> fetch(ClientRequest request) async {
    /*try {
      ConnectivityResult result = await Connectivity().checkConnectivity();
      if(result == ConnectivityResult.none)
        throw InternetConectionException('Sem conexão com a Internet');

      return returnResponse(createHttpResponse(await request()));

    } on SocketException catch(e){
      throw FetchDataException("Ocorreu um error ao realizar sua requisição, contate o suporte.");
    } catch (e) {
      return returnResponse(createHttpResponse(e));
    }*/

    try {
      ConnectivityResult result = await Connectivity().checkConnectivity();
      if (result == ConnectivityResult.none)
        throw InternetConnectionException();
      return returnResponse(createHttpResponse(await request()));
    } on SocketException {
      throw FetchDataException(
          "Problema ao conectar-se ao servidor. Por favor, tente novamente.");
    } catch (e) {
      return returnResponse(createHttpResponse(e));
    }
  }

  String get apiUrl => _apiUrl;
}
